@@ -22,7 +22,7 @@ namespace netgen
           ArrayMem<int, 1> points;
           tree.GetIntersecting(p, p, points);
           if(points.Size()==0)
-              throw Exception("cannot find mapped point");
+              throw Exception("cannot find mapped point " + ToString(p));
           return points[0];
       }
 
@@ -266,7 +266,7 @@ namespace netgen
         for(auto & ident: f->identifications)
           for(auto e : static_cast<GeometryFace*>(ident.from)->edges)
             for(auto e_other : static_cast<GeometryFace*>(ident.to)->edges)
-              if(e->IsMappedShape(*e_other, ident.trafo, tol))
+              if(ident.trafo && e->IsMappedShape(*e_other, *ident.trafo, tol))
                 e->identifications.Append( {e, e_other, ident.trafo, ident.type, ident.name} );
 
       for(auto & e : edges)
@@ -278,9 +278,11 @@ namespace netgen
               GeometryVertex * pfrom[] = { &from.GetStartVertex(), &from.GetEndVertex() };
               GeometryVertex * pto[] = { &to.GetStartVertex(), &to.GetEndVertex() };
 
+              if(!ident.trafo) continue;
+
               // swap points of other edge if necessary
-              Point<3> p_from0 = ident.trafo(from.GetStartVertex().GetPoint());
-              Point<3> p_from1 = ident.trafo(from.GetEndVertex().GetPoint());
+              Point<3> p_from0 = (*ident.trafo)(from.GetStartVertex().GetPoint());
+              Point<3> p_from1 = (*ident.trafo)(from.GetEndVertex().GetPoint());
               Point<3> p_to0 = to.GetStartVertex().GetPoint();
 
               if(Dist(p_from1, p_to0) < Dist(p_from0, p_to0))
@@ -298,10 +300,7 @@ namespace netgen
       auto find_primary = [&] (auto & shapes)
       {
           for(auto &s : shapes)
-          {
               s->primary = s.get();
-              s->primary_to_me = Transformation<3>{ Vec<3> {0,0,0} }; // init with identity
-          }
 
           bool changed = true;
 
@@ -313,23 +312,21 @@ namespace netgen
               {
                   bool need_inverse = ident.from == s.get();
                   auto other = need_inverse ? ident.to : ident.from;
-                  if(other->nr <= s->primary->nr)
+                  if(other->primary->nr < s->primary->nr)
                   {
-                      auto trafo = ident.trafo;
-                      if(need_inverse)
-                          trafo = trafo.CalcInverse();
-                      s->primary = other;
-                      s->primary_to_me.Combine(trafo, s->primary_to_me);
-                      changed = other->nr != s->primary->nr;
-                  }
-                  if(other->primary->nr <= s->primary->nr)
-                  {
-                      auto trafo = ident.trafo;
-                      if(need_inverse)
-                          trafo = trafo.CalcInverse();
                       s->primary = other->primary;
-                      s->primary_to_me.Combine(trafo, other->primary_to_me);
-                      changed = other->primary->nr != s->primary->nr;
+                      if(ident.trafo)
+                      {
+                        auto trafo = *ident.trafo;
+                        if(need_inverse)
+                            trafo = trafo.CalcInverse();
+                        if(!s->primary_to_me)
+                          s->primary_to_me = Transformation<3>( Vec<3>{0., 0., 0.} );
+                        if(!other->primary_to_me)
+                          other->primary_to_me = Transformation<3>( Vec<3>{0., 0., 0.} );
+                        s->primary_to_me->Combine(trafo, *other->primary_to_me);
+                        changed = true;
+                    }
                   }
               }
             }
@@ -485,6 +482,20 @@ namespace netgen
     static Timer tdivide("Divide Edges");
     RegionTimer rt(tdivide);
     // -------------------- DivideEdge -----------------
+    if(properties.partition)
+      {
+        points.SetSize(properties.partition->Size());
+        params.SetSize(properties.partition->Size()+2);
+        params[0] = 0.0;
+        params.Last() = 1.0;
+        for(auto i : Range(properties.partition->Size()))
+          {
+            params[i+1] = (*properties.partition)[i];
+            points[i] = GetPoint(params[i+1]);
+          }
+        return;
+      }
+
     tdivedgesections.Start();
     auto layer = properties.layer;
     double safety = 0.5*(1.-mparam.grading);
@@ -568,12 +579,13 @@ namespace netgen
 
     auto & identifications = mesh.GetIdentifications();
 
-    std::map<size_t, PointIndex> vert2meshpt;
+    Array<PointIndex> vert2meshpt(vertices.Size());
+    vert2meshpt = PointIndex::INVALID;
     for(auto & vert : vertices)
       {
         auto pi = mesh.AddPoint(vert->GetPoint(), vert->properties.layer);
         tree.Insert(mesh[pi], pi);
-        vert2meshpt[vert->GetHash()] = pi;
+        vert2meshpt[vert->nr] = pi;
         mesh[pi].Singularity(vert->properties.hpref);
         mesh[pi].SetType(FIXEDPOINT);
 
@@ -585,8 +597,8 @@ namespace netgen
 
     for(auto & vert : vertices)
         for(auto & ident : vert->identifications)
-            identifications.Add(vert2meshpt[ident.from->GetHash()],
-                                vert2meshpt[ident.to->GetHash()],
+            identifications.Add(vert2meshpt[ident.from->nr],
+                                vert2meshpt[ident.to->nr],
                                 ident.name,
                                 ident.type);
 
@@ -600,8 +612,8 @@ namespace netgen
         auto edge = edges[edgenr].get();
         PointIndex startp, endp;
         // throws if points are not found
-        startp = vert2meshpt.at(edge->GetStartVertex().GetHash());
-        endp = vert2meshpt.at(edge->GetEndVertex().GetHash());
+        startp = vert2meshpt[edge->GetStartVertex().nr];
+        endp = vert2meshpt[edge->GetEndVertex().nr];
 
         // ignore collapsed edges
         if(startp == endp && edge->GetLength() < 1e-10 * bounding_box.Diam())
@@ -652,7 +664,9 @@ namespace netgen
             edge_params.SetSize(np-2);
             for(auto i : Range(np-2))
             {
-                edge_points[i] = trafo(mesh[pnums_primary[i+1]]);
+                edge_points[i] = mesh[pnums_primary[i+1]];
+                if(trafo)
+                  edge_points[i] = (*trafo)(edge_points[i]);
                 EdgePointGeomInfo gi;
                 edge->ProjectPoint(edge_points[i], &gi);
                 edge_params[i] = gi.dist;
@@ -683,7 +697,8 @@ namespace netgen
             {
               for(size_t i : std::vector{0UL, pnums_primary.Size()-1})
               {
-                auto p_mapped = trafo(mesh[pnums_primary[i]]);
+                auto p_mapped = mesh[pnums_primary[i]];
+                if(trafo) p_mapped = (*trafo)(p_mapped);
                 EdgePointGeomInfo gi;
                 edge->ProjectPoint(p_mapped, &gi);
                 params[i] = gi.dist;
@@ -733,10 +748,16 @@ namespace netgen
           if(ident.from == edge.get())
           {
             auto & pnums = all_pnums[edge->nr];
+            if(pnums.Size() < 2) continue; // degenerated edge
             // start and end vertex are already identified
             for(auto pi : pnums.Range(1, pnums.Size()-1))
             {
-                auto pi_other = tree.Find(ident.trafo(mesh[pi]));
+                Point<3> p_other = mesh[pi];
+                if(ident.trafo)
+                  p_other = (*ident.trafo)(mesh[pi]);
+                else
+                  static_cast<GeometryEdge*>(ident.to)->ProjectPoint(p_other, nullptr);
+                auto pi_other = tree.Find(p_other);
                 identifications.Add(pi, pi_other, ident.name, ident.type);
             }
           }
@@ -775,7 +796,10 @@ namespace netgen
         PointIndex pi = vert->nr + 1;
         if(glob2loc[pi] == 0)
           {
-            meshing.AddPoint(mesh[pi], pi);
+            auto gi = face.Project(mesh[pi]);
+            MultiPointGeomInfo mgi;
+            mgi.AddPointGeomInfo(gi);
+            meshing.AddPoint(mesh[pi], pi, &mgi);
             cntp++;
             glob2loc[pi] = cntp;
           }
@@ -851,7 +875,7 @@ namespace netgen
             constexpr int NOT_MAPPED = -1;
             mapped_edges = UNINITIALIZED;
 
-            Transformation<3> trafo;
+            optional<Transformation<3>> trafo;
 
             if(face.IsConnectingCloseSurfaces())
             {
@@ -893,8 +917,6 @@ namespace netgen
                         Element2d sel(4);
                         sel[0] = s[0];
                         sel[1] = s[1];
-                        sel[2] = tree.Find(trafo(mesh[s[1]]));
-                        sel[3] = tree.Find(trafo(mesh[s[0]]));
                         auto gis = sel.GeomInfo();
                         for(auto i : Range(2))
                         {
@@ -902,6 +924,21 @@ namespace netgen
                             gis[i].v = s.epgeominfo[i].v;
                         }
 
+                        Point<3> p2 = mesh[s[1]];
+                        Point<3> p3 = mesh[s[0]];
+                        if(trafo)
+                        {
+                          p2 = (*trafo)(p2);
+                          p3 = (*trafo)(p3);
+                        }
+                        else 
+                        {
+                          edges[mapped_edges[edgenr]]->ProjectPoint(p2, nullptr);
+                          edges[mapped_edges[edgenr]]->ProjectPoint(p3, nullptr);
+                        }
+                        sel[2] = tree.Find(p2);
+                        sel[3] = tree.Find(p3);
+
                         // find mapped segment to set PointGeomInfo correctly
                         Segment s_other;
                         for(auto si_other : p2seg[sel[2]])
@@ -944,11 +981,12 @@ namespace netgen
     }
 
     bool have_identifications = false;
+    std::map<tuple<PointIndex, int>, PointIndex> mapto;
     for(auto & face : faces)
         if(face->primary != face.get())
         {
             have_identifications = true;
-            MapSurfaceMesh(mesh, *face);
+            MapSurfaceMesh(mesh, *face, mapto);
         }
 
     // identify points on faces
@@ -992,7 +1030,8 @@ namespace netgen
                 if(ident.from == face.get())
                     for(auto pi : pi_of_face[face->nr])
                     {
-                        auto pi_other = tree.Find(ident.trafo(mesh[pi]));
+                        auto pi_primary = ident.from->primary->nr == ident.from->nr ? pi : mapto[{pi, ident.to->primary->nr}];
+                        auto pi_other = ident.to->primary->nr == ident.to->nr ? pi_primary : mapto[{pi_primary, ident.to->nr}];
                         mesh_ident.Add(pi, pi_other, ident.name, ident.type);
                     }
             }
@@ -1002,7 +1041,7 @@ namespace netgen
     multithread.task = savetask;
   }
 
-  void NetgenGeometry :: MapSurfaceMesh( Mesh & mesh, const GeometryFace & dst ) const
+  void NetgenGeometry :: MapSurfaceMesh( Mesh & mesh, const GeometryFace & dst, std::map<tuple<PointIndex, int>, PointIndex> & mapto ) const
   {
     static Timer timer("MapSurfaceMesh");
     RegionTimer rt(timer);
@@ -1029,7 +1068,24 @@ namespace netgen
             auto pi = seg[i];
             if(!is_point_in_tree[pi])
             {
-              tree.Insert(trafo(mesh[pi]), pi);
+              auto p = mesh[pi];
+              if(trafo)
+                p = (*trafo)(p);
+              else
+                for(auto& edge: dst.edges)
+                  if (edge->primary->nr == seg.edgenr-1)
+                    {
+                      if (mesh[pi].Type() == FIXEDPOINT) {
+                        if((edge->GetStartVertex().GetPoint() - p).Length2() >\
+                           (edge->GetEndVertex().GetPoint() - p).Length2())
+                          p = edge->GetEndVertex().GetPoint();
+                        else
+                          p = edge->GetStartVertex().GetPoint();
+                      }
+                      else
+                        edge->ProjectPoint(p, nullptr);
+                    }
+              tree.Insert(p, pi);
               is_point_in_tree[pi] = true;
             }
           }
@@ -1057,6 +1113,7 @@ namespace netgen
     }
 
     xbool do_invert = maybe;
+    if(!trafo) do_invert = true;
 
     // now insert mapped surface elements
     for(auto sei : mesh.SurfaceElements().Range())
@@ -1065,14 +1122,6 @@ namespace netgen
         if(sel.GetIndex() != src.nr+1)
           continue;
 
-        if(do_invert.IsMaybe())
-        {
-            auto n_src = src.GetNormal(mesh[sel[0]]);
-            auto n_dist = dst.GetNormal(trafo(mesh[sel[0]]));
-            Mat<3> normal_matrix;
-            CalcInverse(Trans(trafo.GetMatrix()), normal_matrix);
-            do_invert = (normal_matrix * n_src) * n_dist < 0.0;
-        }
         auto sel_new = sel;
         sel_new.SetIndex(dst.nr+1);
         for(auto i : Range(sel.PNums()))
@@ -1080,60 +1129,75 @@ namespace netgen
             auto pi = sel[i];
             if(!pmap[pi].IsValid())
               {
-                pmap[pi] = mesh.AddPoint(trafo(mesh[pi]), 1, SURFACEPOINT);
+                auto p = mesh[pi];
+                if(trafo)
+                  p = (*trafo)(p);
+                else
+                  dst.Project(p);
+                pmap[pi] = mesh.AddPoint(p, 1, SURFACEPOINT);
               }
               sel_new[i] = pmap[pi];
+              mapto[{pi, dst.nr}] = pmap[pi];
+              mapto[{pmap[pi], src.nr}] = pi;
           }
-          if(do_invert.IsTrue())
-              sel_new.Invert();
+        if(do_invert.IsMaybe())
+        {
+            auto n_src = src.GetNormal(mesh[sel[0]]);
+            auto n_dist = dst.GetNormal(mesh[sel_new[0]]);
+            Mat<3> normal_matrix;
+            CalcInverse(Trans(trafo->GetMatrix()), normal_matrix);
+            do_invert = (normal_matrix * n_src) * n_dist < 0.0;
+        }
+        if(do_invert.IsTrue())
+            sel_new.Invert();
 
-          for(auto i : Range(sel.PNums()))
-          {
-              auto pi = sel_new[i];
-              if(uv_values.Range().Next() <= pi)
-              {
-                  // new point (inner surface point)
-                  PointGeomInfo gi;
-                  dst.CalcPointGeomInfo(mesh[sel_new[i]], gi);
-                  sel_new.GeomInfo()[i] = gi;
-                  continue;
-              }
+        for(auto i : Range(sel.PNums()))
+        {
+            auto pi = sel_new[i];
+            if(uv_values.Range().Next() <= pi)
+            {
+                // new point (inner surface point)
+                PointGeomInfo gi;
+                dst.CalcPointGeomInfo(mesh[sel_new[i]], gi);
+                sel_new.GeomInfo()[i] = gi;
+                continue;
+            }
 
-              const auto & uvs = uv_values[pi];
-              if(uvs.Size() == 1)
-              {
-                  // appears only once -> take uv values from edgepointgeominfo
-                  const auto & [u,v] = uvs[0];
-                  PointGeomInfo gi;
-                  gi.u = u;
-                  gi.v = v;
-                  sel_new.GeomInfo()[i] = gi;
-              }
-              else if(uvs.Size() > 1)
-              {
-                  // multiple uv pairs -> project to close point and select closest uv pair
-                  double eps = 1e-3;
-                  auto p = Point<3>((1.0-eps)*Vec<3>(mesh[sel_new.PNumMod(i+1)]) +
-                                        eps/2*Vec<3>(mesh[sel_new.PNumMod(i+2)]) +
-                                        eps/2*Vec<3>(mesh[sel_new.PNumMod(i+3)]));
-                  PointGeomInfo gi_p, gi;
-                  dst.CalcPointGeomInfo(p, gi_p);
-                  gi.trignum = gi_p.trignum;
-                  double min_dist = numeric_limits<double>::max();
-                  for(const auto & [u,v] : uvs)
-                  {
-                      double dist = (gi_p.u-u)*(gi_p.u-u) + (gi_p.v-v)*(gi_p.v-v);
-                      if(dist < min_dist)
-                      {
-                          min_dist = dist;
-                          gi.u = u;
-                          gi.v = v;
-                      }
-                  }
-                  sel_new.GeomInfo()[i] = gi;
-              }
-              else
-                  throw Exception(string(__FILE__) + ":"+ToString(__LINE__) + " shouldn't come here");
+            const auto & uvs = uv_values[pi];
+            if(uvs.Size() == 1)
+            {
+                // appears only once -> take uv values from edgepointgeominfo
+                const auto & [u,v] = uvs[0];
+                PointGeomInfo gi;
+                gi.u = u;
+                gi.v = v;
+                sel_new.GeomInfo()[i] = gi;
+            }
+            else if(uvs.Size() > 1)
+            {
+                // multiple uv pairs -> project to close point and select closest uv pair
+                double eps = 1e-3;
+                auto p = Point<3>((1.0-eps)*Vec<3>(mesh[sel_new.PNumMod(i+1)]) +
+                                      eps/2*Vec<3>(mesh[sel_new.PNumMod(i+2)]) +
+                                      eps/2*Vec<3>(mesh[sel_new.PNumMod(i+3)]));
+                PointGeomInfo gi_p, gi;
+                dst.CalcPointGeomInfo(p, gi_p);
+                gi.trignum = gi_p.trignum;
+                double min_dist = numeric_limits<double>::max();
+                for(const auto & [u,v] : uvs)
+                {
+                    double dist = (gi_p.u-u)*(gi_p.u-u) + (gi_p.v-v)*(gi_p.v-v);
+                    if(dist < min_dist)
+                    {
+                        min_dist = dist;
+                        gi.u = u;
+                        gi.v = v;
+                    }
+                }
+                sel_new.GeomInfo()[i] = gi;
+            }
+            else
+                throw Exception(string(__FILE__) + ":"+ToString(__LINE__) + " shouldn't come here");
           }
           mesh.AddSurfaceElement(sel_new);
       }
@@ -1271,9 +1335,6 @@ namespace netgen
         if (res != MESHING3_OK) return 1;
         if (multithread.terminate) return 0;
 
-        RemoveIllegalElements (*mesh);
-        if (multithread.terminate) return 0;
-
         MeshQuality3d (*mesh);
       }
 
