@@ -675,6 +675,12 @@ namespace netgen
                 moved_surfaces.SetBit(i);
                 mesh.SetBCName(new_si-1, "mapped_" + name);
               }
+            // curving of surfaces with boundary layers will often
+            // result in pushed through elements, since we do not (yet)
+            // curvature through layers.
+            // Therefore we disable curving for these surfaces.
+            if(!params.keep_surfaceindex)
+              mesh.GetFaceDescriptor(i).SetSurfNr(-1);
           }
       }
 
@@ -985,10 +991,12 @@ namespace netgen
           //   return true;
           // return false;
             auto segs = topo.GetVertexSegments(pi);
+            if(segs.Size() == 1)
+              return true;
             auto first_edgenr = mesh[segs[0]].edgenr;
             for(auto segi : segs)
-                if(mesh[segi].edgenr != first_edgenr)
-                    return true;
+              if(mesh[segi].edgenr != first_edgenr)
+                return true;
             return false;
         };
 
@@ -1222,6 +1230,9 @@ namespace netgen
                 if(surfacefacs[sel.GetIndex()] > 0) Swap(points[0], points[2]);
                 for(auto i : Range(points))
                   el[sel.PNums().Size() + i] = points[i];
+                auto new_index = new_mat_nrs[sel.GetIndex()];
+                if(new_index == -1)
+                  throw Exception("Boundary " + ToString(sel.GetIndex()) + " with name " + mesh.GetBCName(sel.GetIndex()-1) + " extruded, but no new material specified for it!");
                 el.SetIndex(new_mat_nrs[sel.GetIndex()]);
                 mesh.AddVolumeElement(el);
               }
