@@ -25,7 +25,7 @@ namespace netgen
   class NetgenGeometry;
   using namespace std;
 
-  static constexpr int  MPI_TAG_MESH = 210;
+  static constexpr int  NG_MPI_TAG_MESH = 210;
   
 
   enum resthtype { RESTRICTH_FACE, RESTRICTH_EDGE, 
@@ -699,6 +699,9 @@ namespace netgen
     auto & GetCommunicator() const { return this->comm; }
     void SetCommunicator(NgMPI_Comm acomm);
     
+    DLL_HEADER void SplitFacesByAdjacentDomains();
+    DLL_HEADER shared_ptr<Mesh> GetSubMesh(string domains="", string faces="") const;
+
     ///
     DLL_HEADER void SetMaterial (int domnr, const string & mat);
     ///
@@ -745,7 +748,7 @@ namespace netgen
     { return (bcnr < bcnames.Size() && bcnames[bcnr]) ? bcnames[bcnr] : &default_bc; }
 
 
-    NgArray<string*> & GetRegionNamesCD (int codim);
+    DLL_HEADER NgArray<string*> & GetRegionNamesCD (int codim);
     
     ///
     void ClearFaceDescriptors()
