@@ -1,3 +1,4 @@
+#include "pybind11/pytypes.h"
 #ifdef NG_PYTHON
 
 #include <regex>
@@ -24,52 +25,6 @@ public:
 };
 
 
-#ifdef NG_MPI4PY
-#include <mpi4py.h>
-
-struct mpi4py_comm {
-  mpi4py_comm() = default;
-  mpi4py_comm(MPI_Comm value) : value(value) {}
-  operator MPI_Comm () { return value; }
-
-  MPI_Comm value;
-};
-
-namespace pybind11 { namespace detail {
-  template <> struct type_caster<mpi4py_comm> {
-    public:
-    PYBIND11_TYPE_CASTER(mpi4py_comm, _("mpi4py_comm"));
-
-      // Python -> C++
-      bool load(handle src, bool) {
-        PyObject *py_src = src.ptr();
-        // Check that we have been passed an mpi4py communicator
-        if (PyObject_TypeCheck(py_src, &PyMPIComm_Type)) {
-          // Convert to regular MPI communicator
-          value.value = *PyMPIComm_Get(py_src);
-        } else {
-          return false;
-        }
-
-        return !PyErr_Occurred();
-      }
-
-      // C++ -> Python
-      static handle cast(mpi4py_comm src,
-                         return_value_policy /* policy */,
-                         handle /* parent */)
-      {
-        // Create an mpi4py handle
-        return PyMPIComm_New(src.value);
-      }
-  };
-}} // namespace pybind11::detail
-
-#endif // NG_MPI4PY
-
-
-
-
 
 using namespace netgen;
 
@@ -101,9 +56,6 @@ static Transformation<3> global_trafo(Vec<3> (0,0,0));
 
 DLL_HEADER void ExportNetgenMeshing(py::module &m) 
 {
-#ifdef NG_MPI4PY
-  import_mpi4py();
-#endif // NG_MPI4PY
   py::register_exception<NgException>(m, "NgException");
   m.attr("_netgen_executable_started") = py::cast(netgen::netgen_executable_started);
   string script;
@@ -131,51 +83,12 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
 
   py::implicitly_convertible<int, Identifications::ID_TYPE>();
 
-  py::class_<NgMPI_Comm> (m, "MPI_Comm")
-#ifdef NG_MPI4PY
-    .def(py::init([] (mpi4py_comm comm)
-                  {
-                    return NgMPI_Comm(comm);
-                  }))
-    .def_property_readonly ("mpi4py", [] (NgMPI_Comm comm) { return mpi4py_comm(comm); })
-#endif // NG_MPI4PY
-    .def_property_readonly ("rank", &NgMPI_Comm::Rank)
-    .def_property_readonly ("size", &NgMPI_Comm::Size)
-    .def("Barrier", &NgMPI_Comm::Barrier)
-    
-#ifdef PARALLEL
-    .def("WTime", [](NgMPI_Comm  & c) { return MPI_Wtime(); })
-#else
-    .def("WTime", [](NgMPI_Comm  & c) { return -1.0; })
-#endif
-    .def("Sum", [](NgMPI_Comm  & c, double x) { return c.AllReduce(x, MPI_SUM); })
-    .def("Min", [](NgMPI_Comm  & c, double x) { return c.AllReduce(x, MPI_MIN); })
-    .def("Max", [](NgMPI_Comm  & c, double x) { return c.AllReduce(x, MPI_MAX); })
-    .def("Sum", [](NgMPI_Comm  & c, int x) { return c.AllReduce(x, MPI_SUM); })
-    .def("Min", [](NgMPI_Comm  & c, int x) { return c.AllReduce(x, MPI_MIN); })
-    .def("Max", [](NgMPI_Comm  & c, int x) { return c.AllReduce(x, MPI_MAX); })
-    .def("Sum", [](NgMPI_Comm  & c, size_t x) { return c.AllReduce(x, MPI_SUM); })
-    .def("Min", [](NgMPI_Comm  & c, size_t x) { return c.AllReduce(x, MPI_MIN); })
-    .def("Max", [](NgMPI_Comm  & c, size_t x) { return c.AllReduce(x, MPI_MAX); })
-    .def("SubComm", [](NgMPI_Comm & c, std::vector<int> proc_list) {
-        Array<int> procs(proc_list.size());
-        for (int i = 0; i < procs.Size(); i++)
-          { procs[i] = proc_list[i]; }
-        if (!procs.Contains(c.Rank()))
-          { throw Exception("rank "+ToString(c.Rank())+" not in subcomm"); }
-	return c.SubCommunicator(procs);
-      }, py::arg("procs"));
-  ;
-
-
-#ifdef NG_MPI4PY
-  py::implicitly_convertible<mpi4py_comm, NgMPI_Comm>();
-#endif // NG_MPI4PY
-
   
   py::class_<NGDummyArgument>(m, "NGDummyArgument")
     .def("__bool__", []( NGDummyArgument &self ) { return false; } )
     ;
+
+  py::class_<LocalH, shared_ptr<LocalH>>(m, "LocalH");
   
   py::class_<Point<2>> (m, "Point2d")
     .def(py::init<double,double>())
@@ -466,7 +379,11 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
         py::detail::field_descriptor {
           "refine", data_layout["refine"], sizeof(bool),
           py::format_descriptor<bool>::format(),
-          py::detail::npy_format_descriptor<bool>::dtype() }            
+          py::detail::npy_format_descriptor<bool>::dtype() },            
+        py::detail::field_descriptor {
+          "curved", data_layout["curved"], sizeof(bool),
+          py::format_descriptor<bool>::format(),
+          py::detail::npy_format_descriptor<bool>::dtype()}            
       });
   }
 
@@ -557,6 +474,10 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
         py::detail::field_descriptor {
           "refine", data_layout["refine"], sizeof(bool),
           py::format_descriptor<bool>::format(),
+          py::detail::npy_format_descriptor<bool>::dtype() },
+        py::detail::field_descriptor {
+          "curved", data_layout["curved"], sizeof(bool),
+          py::format_descriptor<bool>::format(),
           py::detail::npy_format_descriptor<bool>::dtype() }
       });
   }
@@ -1005,6 +926,24 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
       return FlatArray<int>(self.mlparentsurfaceelement.Size(),
                             &self.mlparentsurfaceelement[0]);
     }, py::keep_alive<0,1>())
+    .def_property_readonly("macromesh", [](Mesh & self) {
+      auto coarsemesh = make_shared<Mesh>();
+      *coarsemesh = *self.coarsemesh;
+      return coarsemesh;
+    }, "mesh before hp-refinement")
+    .def("MacroElementNr", [](Mesh & self, int elnr, optional<int> dim) {
+      // cout << "hpels = " << self.hpelements->Size() << endl;
+      // return self[ElementIndex(elnr)].GetHpElnr();
+      if (!dim) dim = self.GetDimension();
+      switch (*dim)
+        {
+        case 2:
+          return (*self.hpelements)[self[SurfaceElementIndex(elnr)].GetHpElnr()].coarse_elnr;
+        case 3:
+          return (*self.hpelements)[self[ElementIndex(elnr)].GetHpElnr()].coarse_elnr;
+        }
+      throw Exception ("MacroElementNr not implemented for dim");
+    }, py::arg("elnr"), py::arg("dim")=nullopt, "number of macro element of element number elnr")
     .def("FaceDescriptor", static_cast<FaceDescriptor&(Mesh::*)(int)> (&Mesh::GetFaceDescriptor),
          py::return_value_policy::reference)
     .def("GetNFaceDescriptors", &Mesh::GetNFD)
@@ -1250,6 +1189,8 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
 
     .def ("GetCD3Name", &Mesh::GetCD3Name)
     .def ("SetCD3Name", &Mesh::SetCD3Name)
+    .def ("SplitFacesByAdjacentDomains", &Mesh::SplitFacesByAdjacentDomains)
+    .def ("GetSubMesh", &Mesh::GetSubMesh, py::arg("domains")="", py::arg("faces")="")
     .def("GetIdentifications", [](Mesh & self) -> py::list
          {
            py::list points;
@@ -1310,7 +1251,11 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
             else mp.optsteps3d = 5;
             OptimizeVolume (mp, self);
           }, py::arg("mp"), py::call_guard<py::gil_scoped_release>())
-
+    .def("SetLocalH",[](Mesh& self, shared_ptr<LocalH> localh, int layer)
+         {
+           self.SetLocalH(localh, layer);
+         }, py::arg("localh"), py::arg("layer")=1)
+    .def("GetLocalH", &Mesh::GetLocalH)
     .def ("OptimizeMesh2d", [](Mesh & self, MeshingParameters* pars, int faceindex)
           {
             self.CalcLocalH(0.5);
@@ -1463,7 +1408,8 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
                               variant<string, int> domain, bool outside,
                               optional<string> project_boundaries,
                               bool grow_edges, bool limit_growth_vectors,
-                              bool sides_keep_surfaceindex)
+                              bool sides_keep_surfaceindex,
+                              bool keep_surfaceindex)
            {
              BoundaryLayerParameters blp;
              BitArray boundaries(self.GetNFD()+1);
@@ -1549,12 +1495,14 @@ DLL_HEADER void ExportNetgenMeshing(py::module &m)
              blp.grow_edges = grow_edges;
              blp.limit_growth_vectors = limit_growth_vectors;
              blp.sides_keep_surfaceindex = sides_keep_surfaceindex;
+             blp.keep_surfaceindex = keep_surfaceindex;
 
              GenerateBoundaryLayer (self, blp);
              self.UpdateTopology();
            }, py::arg("boundary"), py::arg("thickness"), py::arg("material"),
           py::arg("domains") = ".*", py::arg("outside") = false,
           py::arg("project_boundaries")=nullopt, py::arg("grow_edges")=true, py::arg("limit_growth_vectors") = true, py::arg("sides_keep_surfaceindex")=false,
+          py::arg("keep_surfaceindex")=false,
           R"delimiter(
 Add boundary layer to mesh.
 
