@@ -11,7 +11,6 @@
 #include <BRepGProp.hxx>
 #include <BRepLProp_CLProps.hxx>
 #include <BRepLProp_SLProps.hxx>
-#include <BRepMesh_IncrementalMesh.hxx>
 #include <BRepTools.hxx>
 #include <GProp_GProps.hxx>
 #include <Quantity_Color.hxx>
@@ -654,6 +653,7 @@ namespace netgen
 
         int nfaces = geom.fmap.Extent();
 
+        BuildTriangulation(geom.shape);
         for (int i = 1; i <= nfaces && !multithread.terminate; i++)
           {
             multithread.percent = 100 * (i-1)/double(nfaces);
@@ -663,13 +663,14 @@ namespace netgen
             Handle(Geom_Surface) surf = BRep_Tool::Surface (face);
             Handle(Poly_Triangulation) triangulation = BRep_Tool::Triangulation (face, loc);
 
-            if (triangulation.IsNull())
+            if(triangulation.IsNull())
               {
-                BRepTools::Clean (geom.shape);
-                BRepMesh_IncrementalMesh (geom.shape, 0.01, true);
-                triangulation = BRep_Tool::Triangulation (face, loc);
+                if (geom.shape.Infinite())
+                  throw Exception("Cannot generate mesh for an infinite geometry");
+                else
+                  throw Exception("OCC-Triangulation could not be built");
               }
-
+            
             BRepAdaptor_Surface sf(face, Standard_True);
             // one prop for evaluating and one for derivatives
             BRepLProp_SLProps prop(sf, 0, 1e-5);
