@@ -23,7 +23,6 @@
 #include <BRepExtrema_DistShapeShape.hxx>
 #include <BRepGProp.hxx>
 #include <BRepLib.hxx>
-#include <BRepMesh_IncrementalMesh.hxx>
 #include <BRepOffsetAPI_Sewing.hxx>
 #include <BRepTools.hxx>
 #include <IGESCAFControl_Reader.hxx>
@@ -1262,6 +1261,24 @@ namespace netgen
           }
       }
 
+      // Propagate maxh to children
+      for(auto& solid : solids)
+      {
+        auto& shape = static_cast<OCCSolid&>(*solid).GetShape();
+        if(!OCCGeometry::HaveProperties(shape))
+          continue;
+        for(auto& f : GetFaces(shape))
+        {
+          auto& face = GetFace(f);
+          face.properties.maxh = min2(face.properties.maxh,
+                                      GetProperties(shape).maxh);
+        }
+      }
+      for(auto& face : faces)
+        for(auto& edge : face->edges)
+          edge->properties.maxh = min2(edge->properties.maxh,
+                                       face->properties.maxh);
+
       // Add identifications
       auto add_identifications = [&](auto & shapes, auto & shape_map)
       {
@@ -1387,12 +1404,9 @@ namespace netgen
 
    void OCCGeometry :: BuildVisualizationMesh (double deflection)
    {
-      cout << "Preparing visualization (deflection = " << deflection << ") ... " << flush;
-
-      BRepTools::Clean (shape);
-      // BRepMesh_IncrementalMesh::
-      BRepMesh_IncrementalMesh (shape, deflection, true);
-      cout << "done" << endl;
+      // cout << IM(5) << "Preparing visualization (deflection = " << deflection << ") ... " << flush;
+      BuildTriangulation(shape);
+      // cout << IM(5) << "done" << endl;
    }
 
 
@@ -1714,13 +1728,6 @@ namespace netgen
         BRepTools::Read(shape, ss, builder);
       }
 
-    /*
-    // enumerate shapes and archive only integers
-    auto my_hash = [](const TopoDS_Shape & key) {
-        auto occ_hash = key.HashCode(1<<31UL);
-        return std::hash<decltype(occ_hash)>()(occ_hash);
-    };
-    */
     TopTools_IndexedMapOfShape shape_map;
     Array<TopoDS_Shape> shape_list;
 
@@ -1788,7 +1795,7 @@ namespace netgen
    "Face", "Wire", "Edge", "Vertex"};
 
   const char * orientationstring[] =
-     {"+", "-"};
+    {"+", "-", "i", "e"};
 
 
 
@@ -1847,6 +1854,16 @@ namespace netgen
          }
 
          str << "{" << shapename[l] << " " << count2;
+         if(HaveProperties(e.Current()))
+           {
+             const auto& props = GetProperties(e.Current());
+             if(props.name || props.maxh < 1e99)
+               str << " - ";
+             if(props.name)
+               str << props.GetName();
+             if(props.maxh < 1e99)
+               str << " maxh(" << props.maxh << ")";
+           }
 
          if (l <= TopAbs_EDGE)
          {
@@ -2342,21 +2359,27 @@ namespace netgen
 
           for(auto & ident : identifications)
           {
+              const auto& to = STEPConstruct::FindEntity(finder, ident.from == shape ? ident.to : ident.from);
+              if(to.IsNull())
+                  continue;
               Array<Handle(StepRepr_RepresentationItem)> items;
-              // items.Append(STEPConstruct::FindEntity(finder, ident.other)); // TODO!
-              auto & m = ident.trafo.GetMatrix();
+              items.Append(MakeReal(ident.from == shape ? 1 : 0));
+              items.Append(to);
+              Transformation<3> trafo;
+              if(ident.trafo) trafo = *ident.trafo;
+              auto & m = trafo.GetMatrix();
               for(auto i : Range(9))
                   items.Append(MakeReal(m(i)));
-              auto & v = ident.trafo.GetVector();
+              auto & v = trafo.GetVector();
               for(auto i : Range(3))
                   items.Append(MakeReal(v(i)));
-              for(auto & item : items.Range(1,items.Size()))
+              items.Append(MakeInt(ident.type));
+              for(auto & item : items.Range(0, items.Size()))
                   model->AddEntity(item);
               ident_items.Append(MakeCompound(items, ident.name));
           }
-
-          for(auto & item : ident_items.Range(1,ident_items.Size()))
-              model->AddEntity(item);
+          for(auto & item : ident_items.Range(1, ident_items.Size()))
+            model->AddEntity(item);
           auto comp = MakeCompound(ident_items, "netgen_geometry_identification");
           model->AddEntity(comp);
       }
@@ -2373,15 +2396,29 @@ namespace netgen
               auto id_item = Handle(StepRepr_CompoundRepresentationItem)::DownCast(idents->ItemElementValue(i));
               OCCIdentification ident;
               ident.name = id_item->Name()->ToCString();
-              // ident.other = TransferBRep::ShapeResult(transProc->Find(id_item->ItemElementValue(1))); /TODO!
+              auto is_from = ReadReal(id_item->ItemElementValue(1));
+              if(is_from)
+                {
+                  ident.from = shape_origin;
+                  ident.to = TransferBRep::ShapeResult(transProc->Find(id_item->ItemElementValue(2)));
+                }
+              else
+                {
+                  ident.from = TransferBRep::ShapeResult(
+                      transProc->Find(id_item->ItemElementValue(2)));
+                  ident.to = shape_origin;
+                }
 
-              auto & m = ident.trafo.GetMatrix();
+              Transformation<3> trafo;
+              auto & m = trafo.GetMatrix();
               for(auto i : Range(9))
                   m(i) = ReadReal(id_item->ItemElementValue(3+i));
-              auto & v = ident.trafo.GetVector();
+              auto & v = trafo.GetVector();
               for(auto i : Range(3))
                   v(i) = ReadReal(id_item->ItemElementValue(12+i));
-
+              if(FlatVector(9, &trafo.GetMatrix()(0,0)).L2Norm() != .0 && trafo.GetVector().Length2() != .0)
+                  ident.trafo = trafo;
+              ident.type = Identifications::ID_TYPE(ReadInt(id_item->ItemElementValue(15)));
               result.push_back(ident);
           }
           OCCGeometry::GetIdentifications(shape_origin) = result;
