@@ -9,6 +9,7 @@
 #include <meshing.hpp>
 
 #include "occgeom.hpp"
+#include "occ_utils.hpp"
 
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdeprecated-declarations"
@@ -32,7 +33,6 @@
 #include <BRepGProp.hxx>
 #include <BRepLProp_SLProps.hxx>
 #include <BRepLib.hxx>
-#include <BRepMesh_IncrementalMesh.hxx>
 #include <BRepOffsetAPI_MakeOffset.hxx>
 #include <BRepOffsetAPI_MakePipe.hxx>
 #include <BRepOffsetAPI_MakePipeShell.hxx>
@@ -45,7 +45,6 @@
 #include <BRepPrimAPI_MakePrism.hxx>
 #include <BRepPrimAPI_MakeRevol.hxx>
 #include <BRepPrimAPI_MakeSphere.hxx>
-#include <BRepTools.hxx>
 #include <GCE2d_MakeArcOfCircle.hxx>
 #include <GCE2d_MakeCircle.hxx>
 #include <GCE2d_MakeSegment.hxx>
@@ -631,13 +630,18 @@ public:
   
   auto Offset(double d)
   {
+    Finish();
     TopoDS_Wire wire = wires.back();
     wires.pop_back();
-    BRepOffsetAPI_MakeOffset builder;
-    builder.AddWire(wire);
+
+    // handle wires containing a single edge correctly, see
+    // https://dev.opencascade.org/content/brepoffsetapimakeoffset-open-topodswire
+    BRepBuilderAPI_MakeFace makeFace{gp_Pln{axes}};
+    makeFace.Add(wire);
+    BRepOffsetAPI_MakeOffset builder(makeFace.Face());
     builder.Perform(d);
     auto shape = builder.Shape();
-    wires.push_back (TopoDS::Wire(shape.Reversed()));
+    wires.push_back (TopoDS::Wire(shape));
     return shared_from_this();
   }
   
@@ -679,6 +683,10 @@ DLL_HEADER void ExportNgOCCShapes(py::module &m)
     .export_values()
     ;
   
+  m.def("ResetGlobalShapeProperties", [] () {
+    OCCGeometry::global_shape_properties.clear();
+    OCCGeometry::global_shape_property_indices.Clear();
+  });
   
   py::class_<TopoDS_Shape> (m, "TopoDS_Shape")
     .def("__str__", [] (const TopoDS_Shape & shape)
@@ -736,6 +744,10 @@ DLL_HEADER void ExportNgOCCShapes(py::module &m)
            return Mass(shape);
       }, "returns mass of shape, what is length, face, or volume")
 
+    .def_property_readonly("inertia", [](const TopoDS_Shape & shape) {
+           return Properties(shape).MatrixOfInertia();           
+      }, "returns matrix of inertia of shape")
+    
     .def("Move", [](const TopoDS_Shape & shape, const gp_Vec v)
          {
            // which one to choose ? 
@@ -768,7 +780,7 @@ DLL_HEADER void ExportNgOCCShapes(py::module &m)
            PropagateProperties(builder, shape, occ2ng(trafo));
            return builder.Shape();
          }, py::arg("axes"),
-         "copy shape, and mirror over plane defined by 'axes'")
+         "copy shape, and mirror over XY - plane defined by 'axes'")
     
     .def("Mirror", [] (const TopoDS_Shape & shape, const gp_Ax1 & ax)
          {
@@ -778,7 +790,7 @@ DLL_HEADER void ExportNgOCCShapes(py::module &m)
            PropagateProperties(builder, shape, occ2ng(trafo));
            return builder.Shape();
          }, py::arg("axes"),
-         "copy shape, and mirror around axis 'axis'")
+         "copy shape, and rotate by 180 deg around axis 'axis'")
     
     .def("Scale", [](const TopoDS_Shape & shape, const gp_Pnt p, double s)
          {
@@ -824,12 +836,7 @@ DLL_HEADER void ExportNgOCCShapes(py::module &m)
                   },
                   [](TopoDS_Shape& self, double val)
                   {
-                    for (auto typ : { TopAbs_SOLID, TopAbs_FACE,  TopAbs_EDGE, TopAbs_VERTEX })
-                      for (TopExp_Explorer e(self, typ); e.More(); e.Next())
-                      {
-                        auto & maxh = OCCGeometry::GetProperties(e.Current()).maxh;
-                        maxh = min2(val, maxh);
-                      }
+                    OCCGeometry::GetProperties(self).maxh = val;
                   }, "maximal mesh-size for shape")
     
     .def_property("hpref",
@@ -1142,12 +1149,63 @@ DLL_HEADER void ExportNgOCCShapes(py::module &m)
          py::arg("intersection") = false,py::arg("joinType")="arc",
          py::arg("removeIntersectingEdges") = false,
          "makes shell-like solid from faces")
+
+    .def("Offset", [](const TopoDS_Shape & shape, 
+                      double offset, double tol, bool intersection,
+                      string joinT, bool removeIntEdges, optional<string> identification_name) {
+           BRepOffsetAPI_MakeOffsetShape maker;
+           GeomAbs_JoinType joinType;
+           if(joinT == "arc")
+             joinType = GeomAbs_Arc;
+           else if(joinT == "intersection")
+             joinType = GeomAbs_Intersection;
+           else if(joinT == "tangent")
+            joinType = GeomAbs_Tangent;
+           else
+             throw Exception("Only joinTypes 'arc', 'intersection' and 'tangent' exist!");
+           
+           maker.PerformByJoin(shape, offset, tol,
+                               BRepOffset_Skin, intersection,
+                               false, joinType, removeIntEdges);
+
+           // PropagateProperties (maker, shape);
+           for (auto typ : { TopAbs_FACE,  TopAbs_EDGE, TopAbs_VERTEX })
+             for (TopExp_Explorer e(shape, typ); e.More(); e.Next())
+               {
+                 auto s = e.Current();
+                 auto prop = OCCGeometry::GetProperties(s);
+                 for (auto mods : maker.Generated(s))
+                   {
+                     if(OCCGeometry::HaveProperties(s))
+                       {
+                         auto & new_props = OCCGeometry::GetProperties(mods);
+                         new_props.Merge(prop);
+                         if (prop.name) new_props.name = string("offset_")+(*prop.name);
+                       }
+                     if(identification_name)
+                       {
+                         OCCIdentification ident;
+                         ident.from = s;
+                         ident.to = mods;
+                         ident.name = *identification_name;
+                         ident.type = Identifications::CLOSESURFACES;
+                         OCCGeometry::GetIdentifications(s).push_back(ident);
+                       }
+                   }
+               }
+           
+           return maker.Shape();
+       }, py::arg("offset"), py::arg("tol"),
+         py::arg("intersection") = false,py::arg("joinType")="arc",
+         py::arg("removeIntersectingEdges") = false,
+         py::arg("identification_name") = nullopt,
+         "makes shell-like solid from faces")
+
+
     
     .def("MakeTriangulation", [](const TopoDS_Shape & shape)
          {
-           BRepTools::Clean (shape);
-           double deflection = 0.01;
-           BRepMesh_IncrementalMesh (shape, deflection, true);
+           BuildTriangulation(shape);
          })
 
 
@@ -1176,12 +1234,6 @@ DLL_HEADER void ExportNgOCCShapes(py::module &m)
                throw NgException ("Triangulation: shape is not a face");
              }
 
-           /*
-           BRepTools::Clean (shape);
-           double deflection = 0.01;
-           BRepMesh_IncrementalMesh (shape, deflection, true);
-           */
-
            Handle(Geom_Surface) surf = BRep_Tool::Surface (face);
 
            TopLoc_Location loc;
@@ -1189,9 +1241,7 @@ DLL_HEADER void ExportNgOCCShapes(py::module &m)
            
            if (triangulation.IsNull())
              {
-               BRepTools::Clean (shape);
-               double deflection = 0.01;
-               BRepMesh_IncrementalMesh (shape, deflection, true);
+               BuildTriangulation(shape);
                triangulation = BRep_Tool::Triangulation (face, loc);               
              }
            // throw Exception("Don't have a triangulation, call 'MakeTriangulation' first");
@@ -1212,11 +1262,9 @@ DLL_HEADER void ExportNgOCCShapes(py::module &m)
          })
     .def("_webgui_data", [](const TopoDS_Shape & shape)
          {
-           BRepTools::Clean (shape);
-           double deflection = 0.01;
-           BRepMesh_IncrementalMesh (shape, deflection, true);
-           // triangulation = BRep_Tool::Triangulation (face, loc);
-
+           auto status = BuildTriangulation(shape);
+           // cout << "status = " << aStatus << endl;
+           
            std::vector<double> p[3];
            std::vector<double> n[3];
            py::list names, colors, solid_names;
@@ -1402,7 +1450,20 @@ DLL_HEADER void ExportNgOCCShapes(py::module &m)
                              return tuple(s0, s1);
                            },
                            "parameter interval of curve")
-    
+    .def_property("partition",
+       [](TopoDS_Shape & self) -> optional<Array<double>>
+       {
+         if (OCCGeometry::HaveProperties(self))
+           return OCCGeometry::GetProperties(self).partition;
+         return nullopt;
+       },
+       [](TopoDS_Shape &self, py::array_t<double> val)
+       {
+         Array<double> partition(val.size());
+         for(auto i : Range(partition))
+           partition[i] = val.at(i);
+         OCCGeometry::GetProperties(self).partition = std::move(partition);
+       })
     
     .def("Split", [](const TopoDS_Edge& self, py::args args)
     {
@@ -1471,6 +1532,24 @@ DLL_HEADER void ExportNgOCCShapes(py::module &m)
               throw NgException("error in wire builder: "+errstr.str());
             }
         }))
+    .def("Offset", [](const TopoDS_Wire & wire, const TopoDS_Face & face, double dist,
+                      string joinT, bool openresult)
+    {
+      GeomAbs_JoinType joinType;
+      if(joinT == "arc")
+        joinType = GeomAbs_Arc;
+      else if(joinT == "intersection")
+        joinType = GeomAbs_Intersection;
+      else if(joinT == "tangent")
+        joinType = GeomAbs_Tangent;
+      else
+        throw Exception("Only joinTypes 'arc', 'tangent', and 'intersection' exist!");
+      BRepOffsetAPI_MakeOffset builder(face, joinType, openresult);
+      builder.AddWire(wire);
+      builder.Perform(dist);
+      auto shape = builder.Shape();    
+      return shape;
+    })
     ;
 
   py::class_<TopoDS_Face, TopoDS_Shape> (m, "Face")
