@@ -1,8 +1,13 @@
 #include <mystdlib.h> 
-#include <inctcl.hpp>
 #include <meshing.hpp>
+#include <inctcl.hpp>
 #include <core/ngcore_api.hpp>
 
+using std::string;
+using std::endl;
+using std::cout;
+using std::cerr;
+
 namespace netgen
 {
   NGCORE_API_EXPORT Flags parameters;
